@@ -47,6 +47,7 @@ void CmdExtract::DoExtract()
       Cmd->Password.Clean(); // Clean user entered password before processing next archive.
   
     ReconstructDone=false; // Must be reset here, not in ExtractArchiveInit().
+    UseExactVolName=false; // Must be reset here, not in ExtractArchiveInit().
     while (true)
     {
       EXTRACT_ARC_CODE Code=ExtractArchive();
@@ -70,7 +71,7 @@ void CmdExtract::DoExtract()
     // Other error codes may explain a reason of "no files extracted" clearer,
     // so set it only if no other errors found (wrong mask set by user).
     if (ErrHandler.GetErrorCode()==RARX_SUCCESS)
-    ErrHandler.SetErrorCode(RARX_NOFILES);
+      ErrHandler.SetErrorCode(RARX_NOFILES);
   }
   else
     if (!Cmd->DisableDone)
@@ -145,7 +146,7 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive()
     return EXTRACT_ARC_NEXT;
 
 #ifndef SFX_MODULE
-  if (Arc.Volume && !Arc.FirstVolume)
+  if (Arc.Volume && !Arc.FirstVolume && !UseExactVolName)
   {
     wchar FirstVolName[NM];
     VolNameToFirstName(ArcName,FirstVolName,ASIZE(FirstVolName),Arc.NewNumbering);
@@ -163,6 +164,16 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive()
 
   if (Arc.Volume)
   {
+#ifndef SFX_MODULE
+    // Try to speed up extraction for independent solid volumes by starting
+    // extraction from non-first volume if we can.
+    if (!UseExactVolName && Arc.Solid && DetectStartVolume(Arc.FileName,Arc.NewNumbering))
+    {
+      UseExactVolName=true;
+      return EXTRACT_ARC_REPEAT;
+    }
+#endif
+
     // Calculate the total size of all accessible volumes.
     // This size is necessary to display the correct total progress indicator.
 
@@ -323,11 +334,11 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
   Arc.ConvertAttributes();
 
 #if !defined(SFX_MODULE) && !defined(RARDLL)
-  if (Arc.FileHead.SplitBefore && FirstFile)
+  if (Arc.FileHead.SplitBefore && FirstFile && !UseExactVolName)
   {
     wchar CurVolName[NM];
     wcsncpyz(CurVolName,ArcName,ASIZE(CurVolName));
-    VolNameToFirstName(ArcName,ArcName,ASIZE(ArcName),Arc.NewNumbering);
+    GetFirstVolIfFullSet(ArcName,Arc.NewNumbering,ArcName,ASIZE(ArcName));
 
     if (wcsicomp(ArcName,CurVolName)!=0 && FileExist(ArcName))
     {
@@ -591,7 +602,7 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
         TotalFileCount++;
       }
       FileCount++;
-      if (Command!='I')
+      if (Command!='I' && !Cmd->DisableNames)
         if (SkipSolid)
           mprintf(St(MExtrSkipFile),ArcFileName);
         else
@@ -610,8 +621,10 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
               mprintf(St(MExtrFile),DestFileName);
               break;
           }
-      if (!Cmd->DisablePercentage)
+      if (!Cmd->DisablePercentage && !Cmd->DisableNames)
         mprintf(L"     ");
+      if (Cmd->DisableNames)
+        uiEolAfterMsg(); // Avoid erasing preceding messages by percentage indicator in -idn mode.
 
       DataIO.CurUnpRead=0;
       DataIO.CurUnpWrite=0;
@@ -727,7 +740,7 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
       {
         if (ValidCRC)
         {
-          if (Command!='P' && Command!='I')
+          if (Command!='P' && Command!='I' && !Cmd->DisableNames)
             mprintf(L"%s%s ",Cmd->DisablePercentage ? L" ":L"\b\b\b\b\b ",
               Arc.FileHead.FileHash.Type==HASH_NONE ? L"  ?":St(MOk));
         }
@@ -773,14 +786,14 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
           // We could preallocate more space that really written to broken file
           // or file with crafted header.
           if (Preallocated>0 && (BrokenFile || DataIO.CurUnpWrite!=Preallocated))
-          CurFile.Truncate();
+            CurFile.Truncate();
 
 
-        CurFile.SetOpenFileTime(
-          Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.FileHead.mtime,
-          Cmd->xctime==EXTTIME_NONE ? NULL:&Arc.FileHead.ctime,
-          Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.FileHead.atime);
-        CurFile.Close();
+          CurFile.SetOpenFileTime(
+            Cmd->xmtime==EXTTIME_NONE ? NULL:&Arc.FileHead.mtime,
+            Cmd->xctime==EXTTIME_NONE ? NULL:&Arc.FileHead.ctime,
+            Cmd->xatime==EXTTIME_NONE ? NULL:&Arc.FileHead.atime);
+          CurFile.Close();
 
           SetFileHeaderExtra(Cmd,Arc,DestFileName);
 
@@ -797,7 +810,13 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
           Arc.FileHead.FileAttr&=~FILE_ATTRIBUTE_ARCHIVE;
 #endif
         if (!Cmd->IgnoreGeneralAttr && !SetFileAttr(DestFileName,Arc.FileHead.FileAttr))
+        {
           uiMsg(UIERROR_FILEATTR,Arc.FileName,DestFileName);
+          // Android cannot set file attributes and while UIERROR_FILEATTR
+          // above is handled by Android RAR silently, this call would cause
+          // "Operation not permitted" message for every unpacked file.
+          ErrHandler.SysErrMsg();
+        }
 
         PrevProcessed=true;
       }
@@ -890,11 +909,21 @@ void CmdExtract::ExtrPrepareName(Archive &Arc,const wchar *ArcFileName,wchar *De
 #ifndef SFX_MODULE
   if (Cmd->AppendArcNameToPath!=APPENDARCNAME_NONE)
   {
-    if (Cmd->AppendArcNameToPath==APPENDARCNAME_DESTPATH)
-    wcsncatz(DestName,PointToName(Arc.FirstVolumeName),DestSize);
-    else
-      wcsncpyz(DestName,Arc.FirstVolumeName,DestSize); // To archive own dir.
+    switch(Cmd->AppendArcNameToPath)
+    {
+      case APPENDARCNAME_DESTPATH: // To subdir of destination path.
+      wcsncatz(DestName,PointToName(Arc.FirstVolumeName),DestSize);
+        SetExt(DestName,NULL,DestSize);
+        break;
+      case APPENDARCNAME_OWNSUBDIR: // To subdir of archive own dir.
+        wcsncpyz(DestName,Arc.FirstVolumeName,DestSize);
     SetExt(DestName,NULL,DestSize);
+        break;
+      case APPENDARCNAME_OWNDIR:  // To archive own dir.
+        wcsncpyz(DestName,Arc.FirstVolumeName,DestSize);
+        RemoveNameFromPath(DestName);
+        break;
+    }
     AddEndSlash(DestName,DestSize);
   }
 #endif
@@ -1064,8 +1093,11 @@ void CmdExtract::ExtrCreateDir(Archive &Arc,const wchar *ArcFileName)
 {
   if (Cmd->Test)
   {
+    if (!Cmd->DisableNames)
+    {
     mprintf(St(MExtrTestFile),ArcFileName);
     mprintf(L" %s",St(MOk));
+    }
     return;
   }
 
@@ -1102,8 +1134,11 @@ void CmdExtract::ExtrCreateDir(Archive &Arc,const wchar *ArcFileName)
   }
   if (MDCode==MKDIR_SUCCESS)
   {
+    if (!Cmd->DisableNames)
+    {
     mprintf(St(MCreatDir),DestFileName);
     mprintf(L" %s",St(MOk));
+    }
     PrevProcessed=true;
   }
   else
@@ -1157,6 +1192,9 @@ bool CmdExtract::ExtrCreateFile(Archive &Arc,File &CurFile)
       if (!UserReject)
       {
         ErrHandler.CreateErrorMsg(Arc.FileName,DestFileName);
+        if (FileExist(DestFileName) && IsDir(GetFileAttr(DestFileName)))
+          uiMsg(UIERROR_DIRNAMEEXISTS);
+
 #ifdef RARDLL
         Cmd->DllError=ERAR_ECREATE;
 #endif
@@ -1212,3 +1250,104 @@ bool CmdExtract::CheckUnpVer(Archive &Arc,const wchar *ArcFileName)
   }
   return !WrongVer;
 }
+
+
+#ifndef SFX_MODULE
+// To speed up solid volumes extraction, try to find a non-first start volume,
+// which still allows to unpack all files. It is possible for independent
+// solid volumes with solid statistics reset in the beginning.
+bool CmdExtract::DetectStartVolume(const wchar *VolName,bool NewNumbering)
+{
+  wchar *ArgName=Cmd->FileArgs.GetString();
+  Cmd->FileArgs.Rewind();
+  if (ArgName!=NULL && (wcscmp(ArgName,L"*")==0 || wcscmp(ArgName,L"*.*")==0))
+    return false; // No need to check further for * and *.* masks.
+
+  wchar StartName[NM];
+  *StartName=0;
+  
+  // Start search from first volume if all volumes preceding current are available.
+  wchar NextName[NM];
+  GetFirstVolIfFullSet(VolName,NewNumbering,NextName,ASIZE(NextName));
+
+  bool Matched=false;
+  while (!Matched)
+  {
+    Archive Arc(Cmd);
+    if (!Arc.Open(NextName) || !Arc.IsArchive(false) || !Arc.Volume)
+      break;
+
+    bool OpenNext=false;
+    while (Arc.ReadHeader()>0)
+    {
+      Wait();
+
+      HEADER_TYPE HeaderType=Arc.GetHeaderType();
+      if (HeaderType==HEAD_ENDARC)
+      {
+        OpenNext|=Arc.EndArcHead.NextVolume; // Allow open next volume.
+        break;
+      }
+      if (HeaderType==HEAD_FILE)
+      {
+        if (!Arc.FileHead.SplitBefore)
+        {
+          if (!Arc.FileHead.Solid) // Can start extraction from here.
+            wcsncpyz(StartName,NextName,ASIZE(StartName));
+
+          if (Cmd->IsProcessFile(Arc.FileHead,NULL,MATCH_WILDSUBPATH,0,NULL,0)!=0)
+          {
+            Matched=true; // First matched file found, must stop further scan.
+            break;
+          }
+        }
+        if (Arc.FileHead.SplitAfter)
+        {
+          OpenNext=true; // Allow open next volume.
+          break;
+        }
+      }
+      Arc.SeekToNext();
+    }
+    Arc.Close();
+
+    if (!OpenNext)
+      break;
+
+    NextVolumeName(NextName,ASIZE(NextName),!Arc.NewNumbering);
+  }
+  bool NewStartFound=wcscmp(VolName,StartName)!=0;
+  if (NewStartFound) // Found a new volume to start extraction.
+    wcsncpyz(ArcName,StartName,ASIZE(ArcName));
+  
+  return NewStartFound;
+}
+#endif
+
+
+#ifndef SFX_MODULE
+// Return the first volume name if all volumes preceding the specified
+// are available. Otherwise return the specified volume name.
+void CmdExtract::GetFirstVolIfFullSet(const wchar *SrcName,bool NewNumbering,wchar *DestName,size_t DestSize)
+{
+  wchar FirstVolName[NM];
+  VolNameToFirstName(SrcName,FirstVolName,ASIZE(FirstVolName),NewNumbering);
+  wchar NextName[NM];
+  wcsncpyz(NextName,FirstVolName,ASIZE(NextName));
+  wchar ResultName[NM];
+  wcsncpyz(ResultName,SrcName,ASIZE(ResultName));
+  while (true)
+  {
+    if (wcscmp(SrcName,NextName)==0)
+    {
+      wcsncpyz(ResultName,FirstVolName,DestSize);
+      break;
+    }
+    if (!FileExist(NextName))
+      break;
+    NextVolumeName(NextName,ASIZE(NextName),!NewNumbering);
+  }
+  wcsncpyz(DestName,ResultName,DestSize);
+}
+
+#endif
\ No newline at end of file
