@@ -34,7 +34,7 @@ bool MergeArchive(Archive &Arc,ComprDataIO *DataIO,bool ShowFileName,wchar Comma
   Arc.Close();
 
   wchar NextName[NM];
-  wcscpy(NextName,Arc.FileName);
+  wcsncpyz(NextName,Arc.FileName,ASIZE(NextName));
   NextVolumeName(NextName,ASIZE(NextName),!Arc.NewNumbering);
 
 #if !defined(SFX_MODULE) && !defined(RARDLL)
@@ -67,12 +67,12 @@ bool MergeArchive(Archive &Arc,ComprDataIO *DataIO,bool ShowFileName,wchar Comma
         // Checking for new style volumes renamed by user to old style
         // name format. Some users did it for unknown reason.
         wchar AltNextName[NM];
-        wcscpy(AltNextName,Arc.FileName);
+        wcsncpyz(AltNextName,Arc.FileName,ASIZE(AltNextName));
         NextVolumeName(AltNextName,ASIZE(AltNextName),true);
         OldSchemeTested=true;
         if (Arc.Open(AltNextName,OpenMode))
         {
-          wcscpy(NextName,AltNextName);
+          wcsncpyz(NextName,AltNextName,ASIZE(NextName));
           break;
         }
       }
@@ -186,7 +186,7 @@ bool DllVolChange(RAROptions *Cmd,wchar *NextName,size_t NameSize)
   if (Cmd->Callback!=NULL)
   {
     wchar OrgNextName[NM];
-    wcscpy(OrgNextName,NextName);
+    wcsncpyz(OrgNextName,NextName,ASIZE(OrgNextName));
     if (Cmd->Callback(UCM_CHANGEVOLUMEW,Cmd->UserData,(LPARAM)NextName,RAR_VOL_ASK)==-1)
       DllVolAborted=true;
     else
@@ -196,7 +196,7 @@ bool DllVolChange(RAROptions *Cmd,wchar *NextName,size_t NameSize)
       {
         char NextNameA[NM],OrgNextNameA[NM];
         WideToChar(NextName,NextNameA,ASIZE(NextNameA));
-        strcpy(OrgNextNameA,NextNameA);
+        strncpyz(OrgNextNameA,NextNameA,ASIZE(OrgNextNameA));
         if (Cmd->Callback(UCM_CHANGEVOLUME,Cmd->UserData,(LPARAM)NextNameA,RAR_VOL_ASK)==-1)
           DllVolAborted=true;
         else
